/** @file
 * @brief HTTPHelper class implementation. This class represents a Helper to perform HTTP operations easily.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#include "httphelper.h"


/**
 * Constructor. Use this constructor if HTTP AUTH should be used. Username and password will then be set as HTTP auth options.
 * @param _log Logger Object 
 * @param _proxy Proxy to use
 * @param _proxy_port Proxy Port
 * @param _username Username
 * @param _password Password
 */
HTTPHelper::HTTPHelper(Logger::Ptr _log, const string& _proxy, const int _proxy_port, const string& _username, const string& _password)
    : Log(_log)
    , Proxy(_proxy)
    , ProxyPort(_proxy_port)
{
    CurlEasyHandle = init_curl(CurlWritedataBuff, CurlErrBuff);
    set_curl_auth(_username,_password);
}


/**
 * Constructor. Use this constructor if you have to encode the username and password into the url
 * @param _log Logger Object 
 * @param _proxy Proxy to use
 * @param _proxy_port Proxy Port
 */
HTTPHelper::HTTPHelper(Logger::Ptr _log, const string& _proxy, const int _proxy_port)
    : Log(_log)
    , Proxy(_proxy)
    , ProxyPort(_proxy_port)
{
    CurlEasyHandle = init_curl(CurlWritedataBuff, CurlErrBuff);
}


/**
 * Destructor
 */
HTTPHelper::~HTTPHelper()
{
    // Free memory
    curl_easy_cleanup(CurlEasyHandle);
}


/**
 * Perform a HTTP GET operation
 * @param url URL for HTTP GET operation
 * @return The status code from the http operation or -1 if an curl error occurs
 */
long HTTPHelper::http_get(const string& url)
{
    int curl_err_code;
    long curl_info;

    set_curl_url(url);

    if ( (curl_err_code = curl_easy_perform(CurlEasyHandle) ) != 0 )
    {
        Log->print_curl_error(url,curl_err_code,CurlErrBuff);
        return -1;
    }
    if ( (curl_err_code = curl_easy_getinfo(CurlEasyHandle,CURLINFO_RESPONSE_CODE,&curl_info)) != 0 )
    {
        Log->print_curl_error(url,curl_err_code);
        return -1;
    }

    Log->print_curl_data(CurlWritedataBuff);

    // Operation performed without any problems so we can return the curl_info
    return curl_info;
}


/**
 * Getter for member CurlWritedataBuff
 * @return CurlWritedataBuff
 */
string HTTPHelper::get_curl_data() const
{
    return CurlWritedataBuff;
}


/**
 * Initialized curl easy handle with a few options.
 * @param curl_writedata_buff Reference to a string wich will be filled with the curl result
 * @param curl_err_buff A pointer to an char array which will be filled with error message.
 * @return A pointer to the easy curl handle.
 */
CURL* HTTPHelper::init_curl(string& curl_writedata_buff,char* curl_err_buff) const
{
    string user_agent = "Bullet Proof DYNDNS Daemon - Intra2net AG 2009";

    CURL *curl_easy_handle = curl_easy_init();

    curl_easy_setopt(curl_easy_handle,CURLOPT_NOPROGRESS,1);
    curl_easy_setopt(curl_easy_handle,CURLOPT_CONNECTTIMEOUT,5);
    curl_easy_setopt(curl_easy_handle,CURLOPT_TIMEOUT,10);
    curl_easy_setopt(curl_easy_handle,CURLOPT_BUFFERSIZE,1024);
    curl_easy_setopt(curl_easy_handle,CURLOPT_ERRORBUFFER,curl_err_buff);
    curl_easy_setopt(curl_easy_handle,CURLOPT_WRITEFUNCTION,http_receive);
    curl_easy_setopt(curl_easy_handle,CURLOPT_WRITEDATA,&curl_writedata_buff);
    curl_easy_setopt(curl_easy_handle,CURLOPT_USERAGENT,&user_agent);

    if ( !Proxy.empty() )
    {
        curl_easy_setopt(curl_easy_handle,CURLOPT_PROXY,Proxy.c_str());
        curl_easy_setopt(curl_easy_handle,CURLOPT_PROXYPORT,ProxyPort);
    }

    return curl_easy_handle;
}


/**
 * Sets a url to the easy curl handle
 * @param url The url to set.
 */
void HTTPHelper::set_curl_url(const string& url)
{
    curl_easy_setopt(CurlEasyHandle,CURLOPT_URL,url.c_str());
}


/**
 * Sets HTTP AUTH parameters
 * @param username The username for HTTP AUTH
 * @param password The password for HTTP AUTH
 */
void HTTPHelper::set_curl_auth(const string& username, const string& password)
{
    curl_easy_setopt(CurlEasyHandle,CURLOPT_USERNAME,username.c_str());
    curl_easy_setopt(CurlEasyHandle,CURLOPT_PASSWORD,password.c_str());
}


/**
 * Callback Function is called every time CURL is receiving data from HTTPS-Server and will copy all received Data to the given stream pointer
 * @param inBuffer Pointer to input.
 * @param size How many mem blocks are received
 * @param nmemb size of each memblock
 * @param outBuffer Pointer to output stream.
 * @return The size received.
 */
int HTTPHelper::http_receive( char *inBuffer, size_t size, size_t nmemb, string *outBuffer )
{
    outBuffer->append(inBuffer);
    return (size*nmemb);
}

